@@ -19,18 +19,14 @@
 # Boston, MA 02111-1307, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import copy
 import operator
 from itertools import chain
+from collections import OrderedDict
 
 from . import message
 
-from .collections import OrderedDict
+from .sourcescanner import CTYPE_TYPEDEF, CSYMBOL_TYPE_TYPEDEF
 from .message import Position
 from .utils import to_underscores
 
@@ -109,7 +105,9 @@ in contrast to the other create_type() functions."""
                        ctype=fundamental.ctype)
         if gtype_name == 'GHashTable':
             return Map(TYPE_ANY, TYPE_ANY, gtype_name=gtype_name)
-        elif gtype_name in ('GArray', 'GPtrArray', 'GByteArray'):
+        elif gtype_name == 'GByteArray':
+            return Array('GLib.ByteArray', TYPE_UINT8, gtype_name=gtype_name)
+        elif gtype_name in ('GArray', 'GPtrArray'):
             return Array('GLib.' + gtype_name[1:], TYPE_ANY,
                          gtype_name=gtype_name)
         elif gtype_name == 'GStrv':
@@ -200,6 +198,7 @@ class TypeUnknown(Type):
     def __init__(self):
         Type.__init__(self, _target_unknown=True)
 
+
 # Fundamental types, two special ones
 TYPE_NONE = Type(target_fundamental='none', ctype='void')
 TYPE_ANY = Type(target_fundamental='gpointer', ctype='gpointer')
@@ -300,23 +299,13 @@ type_names['void'] = TYPE_NONE
 type_names['signed long long'] = TYPE_LONG_LONG
 # C99 stdint exact width types
 type_names['int8_t'] = TYPE_INT8
-type_names['__int8_t'] = TYPE_INT8
 type_names['uint8_t'] = TYPE_UINT8
-type_names['__uint8_t'] = TYPE_UINT8
 type_names['int16_t'] = TYPE_INT16
-type_names['__int16_t'] = TYPE_INT16
 type_names['uint16_t'] = TYPE_UINT16
-type_names['__uint16_t'] = TYPE_UINT16
 type_names['int32_t'] = TYPE_INT32
-type_names['__int32_t'] = TYPE_INT32
 type_names['uint32_t'] = TYPE_UINT32
-type_names['__uint32_t'] = TYPE_UINT32
 type_names['int64_t'] = TYPE_INT64
-type_names['__int64_t'] = TYPE_INT64
 type_names['uint64_t'] = TYPE_UINT64
-type_names['__uint64_t'] = TYPE_UINT64
-# C99 stdbool
-type_names['bool'] = TYPE_BOOLEAN
 
 # A few additional GLib type aliases
 type_names['guchar'] = TYPE_UINT8
@@ -329,6 +318,8 @@ type_names['gssize'] = TYPE_SSIZE
 type_names['gintptr'] = TYPE_INTPTR
 type_names['guintptr'] = TYPE_UINTPTR
 type_names['gconstpointer'] = TYPE_ANY
+type_names['grefcount'] = TYPE_INT
+type_names['gatomicrefcount'] = TYPE_INT
 
 # We used to support these; continue to do so
 type_names['any'] = TYPE_ANY
@@ -353,7 +344,7 @@ type_names['FILE*'] = TYPE_ANY
 # integers it's easy enough to expand them.
 type_names['size_t'] = type_names['gsize']
 type_names['ssize_t'] = type_names['gssize']
-type_names['time_t'] = TYPE_INT64
+type_names['time_t'] = TYPE_LONG
 type_names['off_t'] = type_names['gsize']
 type_names['pid_t'] = TYPE_INT
 type_names['uid_t'] = TYPE_UINT
@@ -570,6 +561,7 @@ properties."""
         self.deprecated = None
         self.deprecated_doc = None
         self.doc = None
+        self.doc_position = None
 
 
 class Node(Annotated):
@@ -637,9 +629,23 @@ GIName.  It's possible for nodes to contain or point to other nodes."""
     def add_file_position(self, position):
         self.file_positions.add(position)
 
+    def get_main_position(self):
+        if not self.file_positions:
+            return None
+
+        res = None
+        for position in self.file_positions:
+            if position.is_typedef:
+                res = position
+            else:
+                return position
+
+        return res
+
     def add_symbol_reference(self, symbol):
         if symbol.source_filename:
-            self.add_file_position(Position(symbol.source_filename, symbol.line))
+            self.add_file_position(Position(symbol.source_filename, symbol.line,
+                is_typedef=symbol.type in (CTYPE_TYPEDEF, CSYMBOL_TYPE_TYPEDEF)))
 
     def walk(self, callback, chain):
         res = callback(self, chain)
