@@ -20,11 +20,12 @@
 
 import os
 
-from xml.etree.cElementTree import parse
+from collections import OrderedDict
+from xml.etree.ElementTree import parse
 
 from . import ast
 from .girwriter import COMPATIBLE_GIR_VERSION
-from .collections import OrderedDict
+from .message import Position
 
 CORE_NS = "http://www.gtk.org/introspection/core/1.0"
 C_NS = "http://www.gtk.org/introspection/c/1.0"
@@ -184,6 +185,9 @@ class GIRParser(object):
         if doc is not None:
             if doc.text:
                 obj.doc = doc.text
+                obj.doc_position = Position(doc.attrib.get('filename', '<unknown>'),
+                                            doc.attrib.get('line', None),
+                                            doc.attrib.get('column', None))
         version = node.attrib.get('version')
         if version:
             obj.version = version
@@ -214,6 +218,20 @@ class GIRParser(object):
                 attributes_[name] = value
             obj.attributes = attributes_
 
+        if hasattr(obj, 'add_file_position'):
+            positions = sorted(node.findall(_corens('source-position')),
+                               key=lambda x: (x.attrib['filename'],
+                                              int(x.attrib['line'])))
+            for position in positions:
+                if 'column' in position.attrib:
+                    column = int(position.attrib['column'])
+                else:
+                    column = None
+
+                obj.add_file_position(Position(position.attrib['filename'],
+                                               int(position.attrib['line']),
+                                               column))
+
     def _parse_object_interface(self, node):
         parent = node.attrib.get('parent')
         if parent:
@@ -313,7 +331,7 @@ class GIRParser(object):
             raise ValueError('node %r has no return-value' % (name, ))
         transfer = returnnode.attrib.get('transfer-ownership')
         nullable = returnnode.attrib.get('nullable') == '1'
-        retval = ast.Return(self._parse_type(returnnode), nullable, transfer)
+        retval = ast.Return(self._parse_type(returnnode), nullable, False, transfer)
         self._parse_generic_attribs(returnnode, retval)
         parameters = []
 
@@ -366,6 +384,9 @@ class GIRParser(object):
 
         self._parse_type_array_length(parameters, returnnode, retval.type)
 
+        # Re-set the function's parameters to notify it of changes to the list.
+        func.parameters = parameters
+
         self._parse_generic_attribs(node, func)
 
         self._namespace.track(func)
@@ -455,8 +476,8 @@ class GIRParser(object):
                 return ast.Type(ctype=ctype)
             elif name in ['GLib.List', 'GLib.SList']:
                 subchild = self._find_first_child(typenode,
-                                                  map(_corens, ('callback', 'array',
-                                                                'varargs', 'type')))
+                                                  list(map(_corens, ('callback', 'array',
+                                                                '    varargs', 'type'))))
                 if subchild is not None:
                     element_type = self._parse_type(typenode)
                 else:
@@ -464,7 +485,7 @@ class GIRParser(object):
                 return ast.List(name, element_type, ctype=ctype)
             elif name == 'GLib.HashTable':
                 subchildren = self._find_children(typenode, _corens('type'))
-                subchildren_types = map(self._parse_type_simple, subchildren)
+                subchildren_types = list(map(self._parse_type_simple, subchildren))
                 while len(subchildren_types) < 2:
                     subchildren_types.append(ast.TYPE_ANY)
                 return ast.Map(subchildren_types[0], subchildren_types[1], ctype=ctype)
@@ -489,7 +510,7 @@ class GIRParser(object):
         lenidx = typenode.attrib.get('length')
         if lenidx is not None:
             idx = int(lenidx)
-            assert idx < len(siblings), "%r %d >= %d" % (parent, idx, len(siblings))
+            assert idx < len(siblings), "%r %d >= %d" % (siblings, idx, len(siblings))
             if isinstance(siblings[idx], ast.Field):
                 typeval.length_param_name = siblings[idx].name
             else:
@@ -521,7 +542,7 @@ class GIRParser(object):
     def _parse_field(self, node, parent):
         type_node = None
         anonymous_node = None
-        if node.tag in map(_corens, ('record', 'union')):
+        if node.tag in map(_corens, ('callback', 'record', 'union')):
             anonymous_elt = node
         else:
             anonymous_elt = self._find_first_child(node, _corens('callback'))
