@@ -17,11 +17,6 @@
 # Boston, MA 02111-1307, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import re
 
 from . import ast
@@ -35,8 +30,7 @@ from .annotationparser import (ANN_ALLOW_NONE, ANN_ARRAY, ANN_ATTRIBUTES, ANN_CL
                                ANN_VFUNC, ANN_NULLABLE, ANN_OPTIONAL, ANN_NOT)
 from .annotationparser import (OPT_ARRAY_FIXED_SIZE, OPT_ARRAY_LENGTH, OPT_ARRAY_ZERO_TERMINATED,
                                OPT_OUT_CALLEE_ALLOCATES, OPT_OUT_CALLER_ALLOCATES,
-                               OPT_TRANSFER_CONTAINER, OPT_TRANSFER_FLOATING, OPT_TRANSFER_NONE,
-                               OPT_NOT_NULLABLE)
+                               OPT_TRANSFER_CONTAINER, OPT_TRANSFER_FLOATING, OPT_TRANSFER_NONE)
 
 from .utils import to_underscores_noprefix
 
@@ -53,7 +47,7 @@ class MainTransformer(object):
 
     def transform(self):
         if not self._namespace.names:
-            message.fatal('Namespace is empty; likely causes are:\n'
+            message.error('Namespace is empty; likely causes are:\n'
                           '* Not including .h files to be scanned\n'
                           '* Broken --identifier-prefix')
 
@@ -235,8 +229,7 @@ class MainTransformer(object):
             name = self._get_annotation_name(node)
             section_name = 'SECTION:%s' % (name.lower(), )
             block = self._blocks.get(section_name)
-            if block and block.description:
-                node.doc = block.description
+            self._apply_annotations_annotated(node, block)
         if isinstance(node, (ast.Class, ast.Interface)):
             for prop in node.properties:
                 self._apply_annotations_property(node, prop)
@@ -361,14 +354,11 @@ class MainTransformer(object):
                 message.warn("invalid (element-type) for a GPtrArray, "
                              "must be a pointer", annotations.position)
 
-        # GByteArrays have (element-type) guint8 by default
-        if array_type == ast.Array.GLIB_BYTEARRAY:
-            if element_type == ast.TYPE_ANY:
-                array.element_type = ast.TYPE_UINT8
-            elif element_type not in [ast.TYPE_UINT8, ast.TYPE_INT8, ast.TYPE_CHAR]:
-                message.warn("invalid (element-type) for a GByteArray, "
-                             "must be one of guint8, gint8 or gchar",
-                             annotations.position)
+        if (array_type == ast.Array.GLIB_BYTEARRAY
+        and element_type not in [ast.TYPE_UINT8, ast.TYPE_INT8, ast.TYPE_CHAR]):
+            message.warn("invalid (element-type) for a GByteArray, "
+                         "must be one of guint8, gint8 or gchar",
+                         annotations.position)
 
     def _apply_annotations_array(self, parent, node, annotations):
         element_type_options = annotations.get(ANN_ELEMENT_TYPE)
@@ -392,6 +382,7 @@ class MainTransformer(object):
 
         array_options = annotations.get(ANN_ARRAY)
         container_type = ast.Array(array_type, element_type_node, ctype=node.type.ctype,
+                                   complete_ctype=node.type.complete_ctype,
                                    is_const=node.type.is_const)
         if array_options.get(OPT_ARRAY_ZERO_TERMINATED, '0') == '0':
             container_type.zeroterminated = False
@@ -517,7 +508,7 @@ class MainTransformer(object):
                 initially_unowned_type = ast.Type(target_giname='GObject.InitiallyUnowned')
                 try:
                     initially_unowned = self._transformer.lookup_typenode(initially_unowned_type)
-                except KeyError as e:
+                except KeyError:
                     message.error_node(node, "constructor found but GObject is not in includes")
                     return None
                 if initially_unowned and self._is_gi_subclass(typeval, initially_unowned_type):
@@ -666,11 +657,12 @@ class MainTransformer(object):
 
         if ANN_OPTIONAL in annotations:
             if (not isinstance(node, ast.Return) and
-                    node.direction == ast.PARAM_DIRECTION_OUT):
+                    node.direction in [ast.PARAM_DIRECTION_OUT,
+                                       ast.PARAM_DIRECTION_INOUT]):
                 node.optional = True
             else:
                 message.warn('invalid "optional" annotation: '
-                             'only valid for out parameters',
+                             'only valid for out and inout parameters',
                              annotations.position)
 
         if ANN_ALLOW_NONE in annotations:
@@ -696,6 +688,7 @@ class MainTransformer(object):
 
         if tag and tag.description:
             node.doc = tag.description
+            node.doc_position = tag.position
 
         if ANN_SKIP in annotations:
             node.skip = True
@@ -713,6 +706,7 @@ class MainTransformer(object):
 
         if block.description:
             node.doc = block.description
+            node.doc_position = block.position
 
         since_tag = block.tags.get(TAG_SINCE)
         if since_tag is not None:
@@ -862,6 +856,7 @@ class MainTransformer(object):
         if type_annotation:
             field.type = self._transformer.create_type_from_user_string(type_annotation[0])
         field.doc = tag.description
+        field.doc_position = tag.position
         try:
             self._adjust_container_type(parent, field, tag.annotations)
         except AttributeError as ex:
@@ -896,7 +891,7 @@ class MainTransformer(object):
             # the number of parameters (@foo) is the same or greater
             # than the number of signal parameters
             if len(block.params) > len(signal.parameters):
-                names = block.params.items()
+                names = [(k, v) for k, v in block.params.items()]
                 # Resolve real parameter names early, so that in later
                 # phase we can refer to them while resolving annotations.
                 for i, param in enumerate(signal.parameters):
@@ -939,6 +934,7 @@ class MainTransformer(object):
             param = block.params.get(m.symbol, None)
             if param and param.description:
                 m.doc = param.description
+                m.doc_position = param.position
 
     def _pass_read_annotations2(self, node, chain):
         if isinstance(node, ast.Function):
